<?php
/**
 * Plugin Name: MailWizz Export Plugin
 * Description: Export WordPress users to MailWizz with AJAX batching and role selection.
 * Version: 1.1
 * Author: XGEN Technologies Sdn Bhd (www.xgentech.com) for webemail2u.com
 */

// Add settings link on plugin page
add_filter('plugin_action_links_' . plugin_basename(__FILE__), function ($links) {
    $settings_link = '<a href="admin.php?page=mw-export-settings">Settings</a>';
    array_unshift($links, $settings_link);
    return $links;
});

add_action('admin_menu', function () {
    // Add top-level menu
    add_menu_page(
        'MailWizz Export',         // Page title
        'MailWizz Export',         // Menu title
        'manage_options',          // Capability
        'mw-export-settings',      // Menu slug (also used for the settings page)
        'mw_export_settings_page', // Function to render the settings page
        'dashicons-email',         // Icon (optional)
        56                         // Position (optional)
    );

    // Rename the default first submenu to "MailWizz Settings"
    add_submenu_page(
        'mw-export-settings',       // Parent slug
        'MailWizz Settings',        // Page title
        'MailWizz Settings',        // Submenu title
        'manage_options',
        'mw-export-settings',       // Keep this the same as top-level so this loads on first click
        'mw_export_settings_page'
    );

    // Add "Export to MailWizz" submenu
    add_submenu_page(
        'mw-export-settings',
        'Export to MailWizz',
        'Export to MailWizz',
        'manage_options',
        'mw-export-to-mailwizz',
        'mw_export_to_mailwizz_page'
    );
});

// Register plugin settings
add_action('admin_init', function () {
    register_setting('mw_export_settings_group', 'mw_export_api_url');
    register_setting('mw_export_settings_group', 'mw_export_api_key');
    register_setting('mw_export_settings_group', 'mw_export_list_uid');
    register_setting('mw_export_settings_group', 'mw_export_batch_size');
});

// Settings page
function mw_export_settings_page() {
    ?>
    <div class="wrap">
        <h1>MailWizz Export Settings</h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('mw_export_settings_group');
            do_settings_sections('mw-export-settings');
            ?>
            <table class="form-table">
                <tr>
                    <th>MailWizz API URL</th>
                    <td>
                        <input type="text" size="80" name="mw_export_api_url" value="<?php echo esc_attr(get_option('mw_export_api_url', 'https://www.webemail2u.com/mailer/api')); ?>" />
                        <p class="description">
                            This is the base URL of your MailWizz API. For webemail2u.com users, it’s usually: 
                            <code>https://www.webemail2u.com/mailer/api</code>
                        </p>
                    </td>
                </tr>
                <tr>
                    <th>MailWizz API Key</th>
                    <td>
                        <input type="text" size="80" name="mw_export_api_key" value="<?php echo esc_attr(get_option('mw_export_api_key')); ?>" />
                        <p class="description">
                            Your API key from MailWizz. You can get it from your account here:
                            <a href="https://www.webemail2u.com/mailer/customer/api-keys/index" target="_blank">API Keys Page</a>
                        </p>
                    </td>
                </tr>
                <tr>
                    <th>MailWizz List Unique UID</th>
                    <td>
                        <input type="text" size="80" name="mw_export_list_uid" value="<?php echo esc_attr(get_option('mw_export_list_uid')); ?>" />
                        <p class="description">
                            This is the unique identifier for the mailing list where users will be exported to. You can find it in your MailWizz account:
                            <a href="https://www.webemail2u.com/mailer/customer/lists/index" target="_blank">Lists Page</a>
                        </p>
                    </td>
                </tr>
                <tr>
                    <th>Batch Size</th>
                    <td>
                        <input type="number" name="mw_export_batch_size" value="<?php echo esc_attr(get_option('mw_export_batch_size', 100)); ?>" />
                        <p class="description">
                            This defines how many users to process in each batch when exporting. A smaller number helps avoid server timeouts.
                        </p>
                    </td>
                </tr>
            </table>
            <?php submit_button(); ?>
        </form>
    </div>
    <?php
}

// Export Page
function mw_export_to_mailwizz_page() {
    $default_uid = get_option('mw_export_list_uid');
    $default_batch = get_option('mw_export_batch_size', 100);
    ?>
    <div class="wrap">
        <h1>Export Users to MailWizz</h1>

        <table class="form-table">
            <tr>
                <th scope="row"><label for="mw_export_role">User Role</label></th>
                <td>
                    <select id="mw_export_role">
                        <option value="">All Roles</option>
                        <?php 
                        foreach (get_editable_roles() as $role => $details):
                            // Get the count of users for each role
                            $role_count = count_users_in_role($role);
                            ?>
                            <option value="<?php echo esc_attr($role); ?>">
                                <?php echo esc_html($details['name']); ?> (<?php echo $role_count; ?>)
                            </option>
                        <?php endforeach; ?>
                    </select>
                    <p class="description">Select the user role to export. Leave blank to export users of all roles.</p>
                </td>
            </tr>

            <tr>
                <th scope="row"><label for="temporary_uid">List Unique UID</label></th>
                <td>
                    <input type="text" id="temporary_uid" value="<?php echo esc_attr($default_uid); ?>" size="40" />
                    <p class="description">
                        This overrides the List Unique UID temporarily for this export only.
                        Useful when testing or sending to a different list. Original setting remains unchanged.
                    </p>
                </td>
            </tr>

            <tr>
                <th scope="row"><label for="batch_size">Batch Size</label></th>
                <td>
                    <input type="number" id="batch_size" value="<?php echo esc_attr($default_batch); ?>" />
                    <p class="description">
                        How many users to process per request. Lower values reduce risk of timeouts. This value won’t change the saved setting.
                    </p>
                </td>
            </tr>
        </table>

        <p>
            <button id="export-button" class="button button-primary">Export Users</button>
        </p>

        <div id="progress-container" style="margin-top:20px; width:50%; background:#eee; height:20px; border-radius:4px;">
            <div id="progress-bar" style="height:100%; width:0; background:#0073aa;"></div>
        </div>
        <p id="progress-text" style="margin-top:10px;"></p>
    </div>
    <?php
}

// Function to count users in a specific role
function count_users_in_role($role) {
    // Get all users of this role
    $users = get_users(array('role' => $role));
    
    // Return the count of users
    return count($users);
}

// Enqueue scripts
add_action('admin_enqueue_scripts', function ($hook) {
    if (strpos($hook, 'mw-export') === false) return;

    wp_enqueue_script('mw-export-js', plugin_dir_url(__FILE__) . 'js/mw-export.js', ['jquery'], null, true);
    wp_localize_script('mw-export-js', 'mw_export_data', [
        'ajax_url' => admin_url('admin-ajax.php'),
        'nonce'    => wp_create_nonce('mw_export_nonce')
    ]);
});

// AJAX handler
add_action('wp_ajax_mw_export_users_callback', function () {
    check_ajax_referer('mw_export_nonce', 'nonce');

    $role    = sanitize_text_field($_POST['role']);
    $offset  = intval($_POST['offset']);
    $limit   = intval($_POST['limit']);
    $uid     = sanitize_text_field($_POST['uid']);

    $api_url = get_option('mw_export_api_url');
    $api_key = get_option('mw_export_api_key');

    // Validate the List UID
    $valid_uid = mw_validate_list_uid($api_url, $api_key, $uid);
    if (!$valid_uid) {
        wp_send_json_error(['message' => 'Invalid List Unique UID specified. Please double check that it is correct and that it exists in your Mailwizz Panel']);
        return; // Stop further execution
    }

    $args = [
        'fields' => ['display_name', 'user_email'],
        'number' => $limit,
        'offset' => $offset
    ];
    if ($role) $args['role'] = $role;

    $users = get_users($args);
    $results = [];

    foreach ($users as $user) {
        $data = ['EMAIL' => $user->user_email, 'FNAME' => $user->display_name];
        $results[] = [
            'email' => $user->user_email,
            'response' => mw_send_to_mailwizz($api_url, $api_key, $uid, $data)
        ];
    }

    wp_send_json_success(['batch_count' => count($users), 'results' => $results]);
});

// MailWizz API handler
function mw_send_to_mailwizz($api_url, $api_key, $list_uid, $data) {
    $url = rtrim($api_url, '/') . '/lists/' . $list_uid . '/subscribers';

    $response = wp_remote_post($url, [
        'headers' => [
            'X-MW-PUBLIC-KEY' => $api_key,
            'Accept' => 'application/json'
        ],
        'body' => $data
    ]);

    if (is_wp_error($response)) {
        return ['success' => false, 'message' => $response->get_error_message()];
    }

    $status_code = wp_remote_retrieve_response_code($response);
    $body = wp_remote_retrieve_body($response);
    $decoded = json_decode($body, true);

    if ($status_code < 200 || $status_code >= 300 || (isset($decoded['status']) && $decoded['status'] === 'error')) {
        return ['success' => false, 'message' => $decoded['error'] ?? 'Unknown API error'];
    }

    return ['success' => true, 'message' => 'OK'];
}

// Validate List UID by making an API request
function mw_validate_list_uid($api_url, $api_key, $list_uid) {
    $url = rtrim($api_url, '/') . '/lists/' . $list_uid;
    $response = wp_remote_get($url, [
        'headers' => [
            'X-MW-PUBLIC-KEY' => $api_key,
            'Accept' => 'application/json'
        ]
    ]);

    if (is_wp_error($response)) {
        return false;
    }

    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body, true);

    // Check if the response indicates the List UID is valid
    return isset($data['status']) && $data['status'] === 'success';
}
?>